﻿using System;
using System.Collections.Generic;
using MonoGame.Extended;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace CoronaVirusGame.GuiGame
{
    public class GuiGameDisplay : Game
    {
        private GraphicsDeviceManager _graphics;
        private SpriteBatch _spriteBatch;
        private Texture2D _playerTexture;
        private Texture2D _opponentTexture;
        private Texture2D _background;
        private Texture2D _shelfTexture;
        private Texture2D _shelfVerticalTexture;
        private Texture2D _tpTexture;
        private Texture2D _checkoutTexture;
        private Texture2D _restockerTexture;
        private SpriteFont _scoreFont;
        private SpriteFont _scoreboardFont;
        private Level _level;
        private bool _scaleToRes;
        private RestockerManager _restockerManager;
        private Scoreboard _scoreboard;
        public List<Person> People = new List<Person>(); //Always assume the first person is the player
        public List<Checkout> Checkouts = new List<Checkout>();
        public List<Shelf> Shelves = new List<Shelf>();
        public Shelf SelectedShelf; //May be a list if multiplayer is implemented in the future
        public TimeSpan TimeRemaining;
        public bool TimeLimited
        {
            get
            {
                return _level.TimeLimit != null;
            }
        }
        public GuiGameDisplay(Level level, Point resolution, bool scaleToRes = true)
        {
            _graphics = new GraphicsDeviceManager(this);
            _graphics.PreferredBackBufferWidth = resolution.X;
            _graphics.PreferredBackBufferHeight = resolution.Y;
            _graphics.ApplyChanges();
            Content.RootDirectory = "Content";
            IsMouseVisible = true;
            _level = level;
            _scaleToRes = scaleToRes;
        }
        protected override void Initialize()
        {
            if (TimeLimited)
                TimeRemaining = _level.TimeLimit.Value;
            base.Initialize();
        }
        protected override void LoadContent()
        {
            _spriteBatch = new SpriteBatch(GraphicsDevice);

            _playerTexture = Content.Load<Texture2D>("Player");
            _opponentTexture = Content.Load<Texture2D>("Opponent");
            _background = Content.Load<Texture2D>("Background");
            _shelfTexture = Content.Load<Texture2D>("Shelf");
            _shelfVerticalTexture = Content.Load<Texture2D>("ShelfVertical");
            _tpTexture = Content.Load<Texture2D>("ToiletRoll");
            _checkoutTexture = Content.Load<Texture2D>("Checkout");
            _scoreFont = Content.Load<SpriteFont>("ScoreFont");
            _scoreboardFont = Content.Load<SpriteFont>("ScoreboardFont");
            _restockerTexture = Content.Load<Texture2D>("Restocker");
            int scaleFactor = _graphics.PreferredBackBufferWidth / 16; //Assume for now the aspect ratio is 16:9
            Vector2 scaledStartingPos = new Vector2(_level.StartingPos.X * scaleFactor, _level.StartingPos.Y * scaleFactor);
            Point2 scaledPersonSize = new Point2(_level.PersonSize.X * scaleFactor, _level.PersonSize.Y * scaleFactor);
            float scaledMovementSpeed = _level.MovementSpeed * scaleFactor;
            Point scaledToiletPaperSize = new Point((int)(_level.ToiletPaperSize.X * scaleFactor), (int)(_level.ToiletPaperSize.Y * scaleFactor));
            Random random = new Random();
            Player player = new Player(
                _playerTexture,
                _tpTexture,
                _level.RandomStartingPos ? new Vector2(random.Next(_graphics.PreferredBackBufferWidth), random.Next(_graphics.PreferredBackBufferHeight)) : scaledStartingPos,
                scaledPersonSize,
                scaledMovementSpeed,
                scaledToiletPaperSize
            );
            People.Add(player);
            for (int i = 0; i < _level.OpponentCount; i++)
            {
                OpponentAi toAdd = new OpponentAi
                {
                    Texture = _opponentTexture,
                    Posistion = _level.RandomStartingPos ? new Vector2(random.Next(_graphics.PreferredBackBufferWidth), random.Next(_graphics.PreferredBackBufferHeight)) : scaledStartingPos,
                    PersonSize = scaledPersonSize,
                    TpTexture = _tpTexture,
                    TpPickupCooldown = _level.AiTpPickupCooldown, //TODO: Placeholder
                    MovementSpeed = scaledMovementSpeed,
                    TpSize = scaledToiletPaperSize
                };
                People.Add(toAdd);
            }
            Rectangle scaledCheckout = new RectangleF(_level.Checkout.X * scaleFactor, _level.Checkout.Y * scaleFactor, _level.Checkout.Width * scaleFactor, _level.Checkout.Height * scaleFactor).ToRectangle();
            Checkouts.Add(new Checkout(scaledCheckout)
            { Texture = _checkoutTexture });
            RestockerModel restockerModel = new RestockerModel
            {
                Size = new Point2(_level.RestockerSize.X * scaleFactor, _level.RestockerSize.Y * scaleFactor),
                Texture = _restockerTexture,
                ToiletPaperCapacity = 20,
                TpPickupCooldown = TimeSpan.FromSeconds(0.1)
            };
            _restockerManager = new RestockerManager(this, restockerModel, _level.Restockers, TimeSpan.FromSeconds(5));
            foreach (RectangleF shelfRect in _level.ShelfRectangles)
            {
                Rectangle scaledRect = _scaleToRes 
                    ? new Rectangle(
                        (int)(shelfRect.X * scaleFactor),
                        (int)(shelfRect.Y * scaleFactor),
                        (int)(shelfRect.Width * scaleFactor),
                        (int)(shelfRect.Height * scaleFactor)
                    )
                    : shelfRect.ToRectangle();
                Shelves.Add(
                    new Shelf(scaleFactor, scaledRect)
                    {
                        ShelfTexture = scaledRect.Width > scaledRect.Height ? _shelfVerticalTexture : _shelfTexture,
                        TpTexture = _tpTexture,
                        TpSize = scaledToiletPaperSize,
                        ToiletPaperCount = _level.ToiletPaperPerShelf,
                        ToiletPaperCapacity = _level.ToiletPaperPerShelf
                    }
                );
            }
        }
        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                Exit();

            if (TimeLimited && TimeRemaining > TimeSpan.Zero)
                TimeRemaining -= gameTime.ElapsedGameTime;
            if (!TimeLimited || TimeRemaining > TimeSpan.Zero)
            {
                _restockerManager.Update(gameTime);
                foreach (Person person in People)
                {
                    person.Update(gameTime, this);
                    foreach (Checkout checkout in Checkouts)
                    {
                        if (checkout.IsSelectable(person.Rectangle, out _))
                            checkout.SellToiletPaper(person);
                    }
                }
            }
            else if (_scoreboard == null)
                _scoreboard = new Scoreboard(People, _scoreboardFont, new Point(_graphics.PreferredBackBufferWidth, _graphics.PreferredBackBufferHeight));

            base.Update(gameTime);
        }
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            _spriteBatch.Begin(samplerState: SamplerState.PointClamp);
            _spriteBatch.Draw(
                _background,
                new Rectangle(0, 0, _graphics.PreferredBackBufferWidth, _graphics.PreferredBackBufferHeight),
                Color.White
            );
            foreach (Person person in People)
                person.Draw(_spriteBatch);
            foreach (Shelf shelf in Shelves)
                shelf.Draw(_spriteBatch);
            foreach (Checkout checkout in Checkouts)
                checkout.Draw(_spriteBatch);
            _restockerManager.DrawAllRestockers(_spriteBatch);
            if (SelectedShelf != null && SelectedShelf.Accessable)
            {
                float thickness = (SelectedShelf.DrawRectangle.Width + SelectedShelf.DrawRectangle.Height) * 0.035f;
                _spriteBatch.DrawRectangle(SelectedShelf.DrawRectangle, Color.Red, thickness, 1);
            }
            DrawScores(out float timeY);
            _scoreboard?.Draw(_spriteBatch);
            _spriteBatch.DrawString(_scoreFont, $"Time Remaining: {TimeRemaining.ToString()}", new Vector2(0, timeY), Color.White);
            _spriteBatch.End();

            base.Draw(gameTime);
        }
        private void DrawScores(out float nextY)
        {
            int opponentIndex = 1;
            float y = 0;
            float incrementY = _scoreFont.MeasureString("S").Y;
            foreach (Person person in People)
            {
                Type personType = person.GetType();
                if (personType == typeof(Restocker))
                    continue; //Restockers don't participate in the game
                string identifier = Person.GetIdentifier(ref opponentIndex, personType);
                string scoreToDraw = $"{identifier} score: {person.Score} pounds";
                string potentialScoreToDraw = $"+{person.PotentialAdditionalScore}";
                float potentialScoreX = _scoreFont.MeasureString(scoreToDraw + " ").X;
                _spriteBatch.DrawString(_scoreFont, scoreToDraw, new Vector2(0, y), Color.White);
                Color potentialScoreColour = person.PotentialAdditionalScore == 100 ? Color.Red : Color.LightGreen;
                _spriteBatch.DrawString(_scoreFont, potentialScoreToDraw, new Vector2(potentialScoreX, y), potentialScoreColour);
                y += incrementY;
            }
            nextY = y;
        }
    }
}
